package com.fcbox.bluetooth.core

import android.os.Bundle
import android.os.Parcel
import android.os.Parcelable

/***
 * <pre>   `case MESSAGE_STATE_CHANGE:
 * switch (msg.arg1) {
 * case BluetoothService.STATE_CONNECTED:
 * int arg1
 * int arg2
 *
 * break;
 * case BluetoothService.STATE_CONNECTING:
 * int arg1
 * int arg2
 * break;
 * case BluetoothService.STATE_LISTEN:
 * case BluetoothService.STATE_NONE:
 * int arg1
 * int arg2
 * break;
 * }
 * break;
 * case MESSAGE_WRITE:
 * int arg1
 * int arg2
 * obj--byte[] buffer
 * break;
 * case MESSAGE_READ:
 * int arg1
 * int arg2
 * obj--byte[] buffer
 * break;
 * case MESSAGE_DEVICE_NAME:
 * data
 * break;
 * case MESSAGE_TOAST:
 * data
 * break;
 * case MESSAGE_CONNECTION_LOST:
 * data
 * break;
 * case MESSAGE_UNABLE_CONNECT:
 * data
 * break;
`</pre> *
 */
class CallBackMsg : Parcelable {
    var what = 0
    var arg1 = 0
    var arg2 = 0

    //Parcelable
    var obj: Any? = null
    var data: Bundle? = null

    constructor() {}
    constructor(what: Int, arg1: Int, arg2: Int, obj: Any?, data: Bundle?) {
        this.what = what
        this.arg1 = arg1
        this.arg2 = arg2
        this.obj = obj
        this.data = data
    }

    protected constructor(parcel: Parcel) {
        what = parcel.readInt()
        arg1 = parcel.readInt()
        arg2 = parcel.readInt()
        if (parcel.readInt() != 0) {
            obj = parcel.readParcelable(javaClass.classLoader)
        }
        data = parcel.readBundle()
    }

    override fun writeToParcel(dest: Parcel, flags: Int) {
        dest.writeInt(what)
        dest.writeInt(arg1)
        dest.writeInt(arg2)
        if (obj != null) {
            try {
                val p = obj as Parcelable
                dest.writeInt(1)
                dest.writeParcelable(p, flags)
            } catch (e: ClassCastException) {
                throw RuntimeException(
                    "Can't marshal non-Parcelable objects across processes."
                )
            }
        } else {
            dest.writeInt(0)
        }
        dest.writeBundle(data)
    }

    override fun describeContents(): Int {
        return 0
    }

    override fun toString(): String {
        return "CallBackMsg{" +
                "what=" + what +
                ", arg1=" + arg1 +
                ", arg2=" + arg2 +
                ", obj=" + obj +
                ", data=" + data +
                '}'
    }

    companion object CREATOR :Parcelable.Creator<CallBackMsg>{
            override fun createFromParcel(parcel: Parcel): CallBackMsg? {
                return CallBackMsg(parcel)
            }

            override fun newArray(size: Int): Array<CallBackMsg?> {
                return arrayOfNulls(size)
            }


        @JvmStatic
        fun newInstance(
            what: Int,
            arg1: Int,
            arg2: Int,
            obj: Any?,
            data: Bundle?
        ): CallBackMsg {
            return CallBackMsg(what, arg1, arg2, obj, data)
        }
    }
}