package com.fcbox.printer

import com.fcbox.printer.base.IPrintAdapter
import com.fcbox.printer.bean.InstructionType
import com.fcbox.printer.manager.InstructionAdapterManager
import com.fcbox.printer.manager.TemplateManager
import com.fcbox.printer.template.CompanyBusinessPrinterTemplate

/**
 * @Description:
 * @Author: luobin
 * @Date:   2021/4/28
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class PrintManager private constructor() {

    private var templateManager: TemplateManager = TemplateManager()
    private var printAdapterManager: InstructionAdapterManager = InstructionAdapterManager()

    fun registerTemplate(clazz: Class<out CompanyBusinessPrinterTemplate>): PrintManager {
        templateManager.registerTemplate(clazz)
        return instance
    }

    fun registerAdapter(
        instructionType: InstructionType,
        printAdapter: IPrintAdapter
    ): PrintManager {
        printAdapterManager.registerAdapter(instructionType, printAdapter)
        return instance
    }


    fun getTemplateManager(): TemplateManager {
        return templateManager
    }

    fun getPrintAdapterManager(): InstructionAdapterManager {
        return printAdapterManager
    }

    fun destroy() {
        templateManager.destroy()
    }

    companion object {
        val instance = PrintManager()
    }


}