package com.fcbox.printer.components

import com.fcbox.template.IPrintOpt
import com.fcbox.printer.opt.*
import com.fcbox.template.BaseTemplate
import kotlin.math.ceil

/**
 * @Description: 条形码+text
 * @Author: luobin
 * @Date:   2021/4/28
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class BarCodeNum private constructor(
    var isCenter: Boolean,
    var barX: Double,
    var barY: Double,
    var barCodeType: Int,
    var barRatio: Double,
    var barHeight: Int,
    var barWidth: Int,//窄条的单位宽度
    var textX: Double,
    var textSize: Int,
    var isBold: Boolean,
    var content: String,

) {

    fun toPrintOpt(): List<IPrintOpt> {

        val result = arrayListOf<IPrintOpt>()

        if (isCenter) {
            val alignOpt = AlignOpt.create(AlignOpt.ALIGN_CENTER)
            result.add(alignOpt)
        } else {
            val alignOpt = AlignOpt.create(AlignOpt.ALIGN_LEFT)
            result.add(alignOpt)
        }

        val barCodeOpt =
            BarCodeOpt.create(barCodeType, barWidth, barRatio, barHeight, barX, barY, content)

        result.add(barCodeOpt)

        val textOpt = TextOpt.create(
            textX,
            barY + ceil(barHeight * 1.0 / BaseTemplate.UNIT_RATIO) + 1,
            textSize,
            isBold,
            content
        )

        result.add(textOpt)

        if (isCenter) {
            val alignOpt = AlignOpt.create(AlignOpt.ALIGN_LEFT)
            result.add(alignOpt)
        }

        return result
    }

    data class Builder(
        var isCenter: Boolean = false,
        var barX: Double? = null,
        var barY: Double? = null,
        var barCodeType: Int = OptTable.BARCODE_TYPE_128,
        var barRatio: Double = 2.0,
        var barHeight: Int? = null,
        var barWidth: Int = 2,//窄条的单位宽度
        var textX: Double? = 0.0,
        var textSize: Int? = null,
        var isBold: Boolean = false,
        var content: String? = null,
    ) {
        fun isAlignCenter(isCenter: Boolean) = apply { this.isCenter = isCenter }
        fun barX(barX: Double) = apply { this.barX = barX }
        fun barY(barY: Double) = apply { this.barY = barY }
        fun barCodeType(barCodeType: Int) = apply { this.barCodeType = barCodeType }
        fun barRatio(barRatio: Double) = apply { this.barRatio = barRatio }
        fun barHeight(barHeight: Int) = apply { this.barHeight = barHeight }
        fun barWidth(barWidth: Int) = apply { this.barWidth = barWidth }
        fun textX(textX: Double) = apply { this.textX = textX }
        fun textSize(textSize: Int) = apply { this.textSize = textSize }
        fun isBold(isBold: Boolean) = apply { this.isBold = isBold }
        fun content(content: String) = apply { this.content = content }

        fun build(): BarCodeNum {

            if (barX == null || barY == null || barHeight == null || textSize == null ||
                content == null ) {
                throw IllegalArgumentException("params is null")
            }

            if (barX!! < 0 || barY!! < 0 || barRatio <= 0 || barWidth <= 0 || textX!! < 0 ||
                textSize!! <= 0) {
                throw IllegalArgumentException("illegalArgument")
            }

            return BarCodeNum(
                isCenter = isCenter,
                barX = barX!!,
                barY = barY!!,
                barCodeType = barCodeType,
                barRatio = barRatio,
                barHeight = barHeight!!,
                barWidth = barWidth,
                textX = textX!!,
                textSize = textSize!!,
                isBold = isBold,
                content = content!!,
            )
        }
    }
}