package com.fcbox.printer.device


import com.fcbox.printer.opt.OptTable
import com.fcbox.printer.opt.TextOpt
import com.fcbox.template.BaseTemplate
import com.fcbox.template.IPrintOpt

/**
 * @Description: 一般机型
 * @Author: luobin
 * @Date:   2021/5/10
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class NormalPrinter(template: BaseTemplate) : PrinterDecorator(template) {

    override fun getTemplateBaseLeft(): Int {
        return 2
    }

    override fun getTemplateBaseTop(): Int {
        return 11
    }

    override fun getTemplateRectWidth(): Int {
        return 70
    }

    override fun convert2Opts(): List<IPrintOpt> {
        template.templateBaseLeft=getTemplateBaseLeft()
        template.templateBaseTop=getTemplateBaseTop()
        template.templateRectWidth=getTemplateRectWidth()

        val list = template.convert2Opts()

        list.forEach {
            if (OptTable.OPT_CODE_TEXT == it.optCode) {
                (it as TextOpt).apply {
                    setFontSizeMode(parseSizeMode(getFontType()))
                }
            }
        }
        return list
    }


    private fun parseSizeMode(type: Int): String {
        var mode = ""
        mode = when (type) {
            OptTable.SIZE_16 -> 55.toString() + " " + 0
            OptTable.SIZE_20 -> 3.toString() + " " + 1
            OptTable.SIZE_28 -> 28.toString() + " " + 0
            OptTable.SIZE_32 -> 55.toString() + " " + 1
            OptTable.SIZE_40 -> 4.toString() + " " + 1
            OptTable.SIZE_48 -> 4.toString() + " " + 2
            OptTable.SIZE_56 -> 4.toString() + " " + 3
            OptTable.SIZE_64 -> 24.toString() + " " + 4
            OptTable.SIZE_72 -> 24.toString() + " " + 6
            else -> 8.toString() + " " + 0
        }
        return mode
    }

}