package com.fcbox.printer.opt

import com.fcbox.template.IPrintOpt

/**
 * @Description: 条形码
 * @Author: luobin
 * @Date:   2021/4/26
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class BarCodeOpt : IPrintOpt {
    override var optCode = OptTable.OPT_CODE_BARCODE

    private var barcodeType = 0
    private var width = 0
    private var ratio = 0.0
    private var height = 0
    private var x = 0.0
    private var y = 0.0
    private var number: String? = null


    fun getBarcodeType(): Int {
        return barcodeType
    }

    fun getWidth(): Int {
        return width
    }

    fun getRatio(): Double {
        return ratio
    }

    fun getHeight(): Int {
        return height
    }

    fun getX(): Double {
        return x
    }

    fun getY(): Double {
        return y
    }

    fun getNumber(): String? {
        return number
    }


    fun setBarcodeType(barcodeType: Int) {
        this.barcodeType = barcodeType
    }

    fun setWidth(width: Int) {
        this.width = width
    }

    fun setRatio(ratio: Double) {
        this.ratio = ratio
    }

    fun setHeight(height: Int) {
        this.height = height
    }

    fun setX(x: Double) {
        this.x = x
    }

    fun setY(y: Double) {
        this.y = y
    }

    fun setNumber(number: String?) {
        this.number = number
    }

    override fun setBaseXY(baseX: Int, baseY: Int) {
        setX(x + baseX)
        setY(y + baseY)
    }

    companion object {
        /**
         * @param barcodeType 条码类型 见 OptTable.BARCODE_TYPE_128 定义
         * @param width 窄条码的宽度点数
         * @param ratio 宽条码和窄条码的比率
         * @param height 条码的单位高度
         * @param x 条码开始的X轴坐标
         * @param y 条码开始的Y轴坐标
         * @param number 条码数据
         */
        @JvmStatic
        fun create(
            barcodeType: Int, width: Int, ratio: Double,
            height: Int, x: Double, y: Double, number: String?
        ): BarCodeOpt {
            val opt = BarCodeOpt()
            opt.barcodeType = barcodeType
            opt.width = width
            opt.ratio = ratio
            opt.height = height
            opt.x = x
            opt.y = y
            opt.number = number
            return opt
        }
    }
}