package com.fcbox.printer.opt

import com.fcbox.template.BaseXYAbs
import com.fcbox.template.IPrintOpt
/**
 * @Description: 二维码指令
 * @Author: luobin
 * @Date:   2021/4/26
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class QrCodeOpt : IPrintOpt, BaseXYAbs {
    override var optCode = OptTable.OPT_CODE_QRCODE

    private var x = 0.0
    private var y = 0.0
    private var qrVersion = 0
    private var unitHeight = 0
    var level = 0
    private var content: String? = null


    fun getX(): Double {
        return x
    }

    fun getY(): Double {
        return y
    }

    fun getQrVersion(): Int {
        return qrVersion
    }

    fun getUnitHeight(): Int {
        return unitHeight
    }

    fun getContent(): String? {
        return content
    }

    fun setX(x: Double) {
        this.x = x
    }

    fun setY(y: Double) {
        this.y = y
    }

    fun setQrVersion(qrVersion: Int) {
        this.qrVersion = qrVersion
    }

    fun setUnitHeight(unitHeight: Int) {
        this.unitHeight = unitHeight
    }

    fun setContent(content: String?) {
        this.content = content
    }

    override fun setBaseXY(baseX: Int, baseY: Int) {
        setX(x + baseX)
        setY(y + baseY)
    }

    companion object {
        /**
         * @param x 条码开始的X轴坐标
         * @param y 条码开始的Y轴坐标
         * @param qrVersion QR代码型号版本 版本1的QR码1边由21个码元构成。版本每高一级，构成QR码一边的码元就增加4个
         * @param unitHeight QR码元的高度点数 也是宽度点数
         * @param level 容错率等级 见 OptTable.QRCODE_LEVEL_L 等字体定义
         * @param content 二维码内容
         */
        fun create(
            x: Double, y: Double, qrVersion: Int, unitHeight: Int,
            level: Int, content: String?
        ): QrCodeOpt {
            val opt = QrCodeOpt()
            opt.x = x
            opt.y = y
            opt.qrVersion = qrVersion
            opt.unitHeight = unitHeight
            //        opt.level = level;
            opt.content = content
            return opt
        }
    }
}