package com.fcbox.printer.util

import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.core.util.Consumer
import com.fcbox.bluetooth.core.FcBluetoothManager
import com.fcbox.printer.PrintManager
import com.fcbox.printer.base.RunnableExceptionWrapper
import com.fcbox.printer.bean.TestLabelInfo
import com.fcbox.printer.bean.PrintConfig
import com.fcbox.printer.device.PrinterFactory
import com.fcbox.template.IPrintOpt
import java.util.concurrent.Executors


/**
 * @Description: 面单打印
 * @Author: luobin
 * @Date:   2021/4/26
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
object ExpressPrint {
    private val singleThreadExecutor = Executors.newSingleThreadExecutor()

    fun printData(
        config: PrintConfig = PrintConfig.Builder().build(),
        bean: Any?,
        callBack: Consumer<Int>? = Consumer { }
    ) {
        if (bean == null) {
            return
        }
        printData(config, listOf(bean), callBack)
    }


    fun printData(config: PrintConfig, data: List<Any>, callBack: Consumer<Int>?) {
        if (data.isEmpty()) {
            return
        }

        if (FcBluetoothManager.getInstance().isConnected().not()) {
            Log.e("ExpressPrint", "isConnected false")
            return
        }

        singleThreadExecutor.execute(RunnableExceptionWrapper {

            data.forEach {
                val templateClass =
                    PrintManager.instance.getTemplateManager().getTemplate(
                        expressCompany = config.expressCompany,
                        businessType = config.businessType,
                    )

                if (templateClass == null) {
                    Log.e("ExpressPrint", "templateClass is null")
                    return@forEach
                }

                val template = templateClass.newInstance().apply {
                    setPrintData(it)
                    // TODO: 2021/5/11 check
//                    setPrintConfig(config)
                }

                val adapter = PrintManager.instance.getPrintAdapterManager()
                    .getPrintAdapter(config.instructionType)

                if (adapter == null) {
                    Log.e("ExpressPrint", "adapter is null")
                    return@forEach
                }

                val printerDecorator = PrinterFactory.generatePrinter(config.printerType, template)

                adapter.apply {
                    val dataInMm: List<IPrintOpt> = printerDecorator.convert2Opts()
                    val dataInPx = adapterPix(dataInMm)
                    CommandSendUtil.portSendCmd(parse(dataInPx))
                }
            }

            Handler(Looper.getMainLooper()).post { callBack?.accept(1) }

        })
    }


    fun printTestData() {
        printData(PrintConfig.Builder().build(), getExLabel())
    }

    private fun getExLabel(): TestLabelInfo {
        return TestLabelInfo().apply {
            this.orderId = ""
            this.expressId = "SF6090239432156"
            this.sendName = "陈小花"
            this.sendPhone = "18011111111"
            this.sendAddress = "东滨路4078号永新汇1号楼5楼东滨路4078号永新汇1号楼5楼东滨路4078号永新汇1号楼5楼"
            this.takePerson = "王大宝"
            this.takePhone = "13623449876"
            this.takeTime = "2021-05-01 12:01:01"
            this.callFlag = true
            this.callPhone = "13400000088"
        }

    }


}