package com.fcbox.template

import java.util.*

/**
 * @author: 朱丽君
 * @create: 20-4-17 下午12:05
 * @description: 基础面单模板
 *
 * |————————————————————————————————————————————|
 * |                Logo区                      |
 * |                                            |
 * |———————————————baseBarAreaTop———————————————|
 * |                                            |
 * |                条码区                       |
 * |                                            |
 * |                                            |
 * |——————————————baseExpressTop————————————————|
 * |                运单区                       |
 * |—————————————————baseReceiverTop————————————|
 * |                                            |
 * |                收件人区                     |
 * |————————————————baseSenderTop———————————————|
 * |                寄件人区                     |
 * |                                            |
 * |———————————————baseInfoTop—— ———————————————|
 * |                                            |
 * |                                            |
 * |                                            |
 * |                快递信息区                   |
 * |                                            |
 * |                                            |
 * |                                            |
 * |————————————————————————————————————————————|
 * 所有单位为mm
 */
abstract class BaseTemplate : IPrintTemplate {
    open val EMPTY_OPT: List<IPrintOpt> = ArrayList()

    open var templateLineWidth = 1 //线条的宽

    open var templateBarCodeAreaHeight = 31 //条码区高度

    open var templateExpressCodeAreaHeight = 8 //运单区域

    open var templateReceiverAreaHeight = 14 //收件人区域

    open var templateSendAreaHeight = 12 //寄件人区域

    open var templateInfoAreaHeight = 30 //快递信息区域

    open var templateBaseLeft = 2 //左边间距

    open var templateBaseTop = 11 //顶部间距

    open var templateRectWidth = 70 //表格区域宽度

    open var baseBarAreaTop = 0
    open var baseExpressTop = 0
    open var baseReceiverTop = 0
    open var baseSenderTop = 0
    open var baseInfoTop = 0


    @JvmField
    protected var bean: Any? = null


    /*
    logo区
  */
    abstract fun getLogoOpts(): List<IPrintOpt>

    /*
   条码区
   */
    abstract fun getBarCodeOpts(): List<IPrintOpt>

    /*
   运单区
   */
    abstract fun getExpressCodeOpts(): List<IPrintOpt>

    /*
   收件人区
   */
    abstract fun getReceiverOpts(): List<IPrintOpt>

    /*
   寄件人区
   */
    abstract fun getSenderOpts(): List<IPrintOpt>

    /*
   快递信息区
   */
    abstract fun getInfoOpts(): List<IPrintOpt>


    abstract fun getReadyOpt(): IPrintOpt

    abstract fun getBoxOpt(): IPrintOpt

    abstract fun getEndOpt(): IPrintOpt

    private val allPrintOpt: MutableList<IPrintOpt> = ArrayList()

    override fun convert2Opts(): List<IPrintOpt> {
        if (bean == null) {
            return listOf()
        }

        initSizeParams()

        allPrintOpt.add(getReadyOpt())

        //logo区(第一栏)
        val logoOpts = getLogoOpts()
        if (logoOpts.isEmpty().not()) {
            addPrintOptList(logoOpts, 0)
        }


        allPrintOpt.add(getBoxOpt())


        //条码区
        addPrintOptList(getBarCodeOpts(), baseBarAreaTop)

        //运单区
        val expressCodeOpts = getExpressCodeOpts()
        if (expressCodeOpts.isEmpty().not()) {
            addPrintOptList(expressCodeOpts, baseExpressTop)
        }

        //收件人区
        val receiverOpts = getReceiverOpts()
        if (receiverOpts.isEmpty().not()) {
            addPrintOptList(receiverOpts, baseReceiverTop)
        }


        //寄件人区
        val senderOpts = getSenderOpts()
        if (senderOpts.isEmpty().not()) {
            addPrintOptList(senderOpts, baseSenderTop)
        }


        //info
        val infoOpts = getInfoOpts()
        if (infoOpts.isEmpty().not()) {
            addPrintOptList(infoOpts, baseInfoTop)
        }

        allPrintOpt.add(getEndOpt())

        return allPrintOpt
    }

    private fun addPrintOptList(printOptList: List<IPrintOpt>?, baseAlignY: Int) {
        if (printOptList.isNullOrEmpty()) {
            return
        }

        printOptList.forEach {
            it.setBaseXY(templateBaseLeft, baseAlignY)
            allPrintOpt.add(it)
        }
    }

    fun setPrintData(bean: Any?): BaseTemplate {
        this.bean = bean
        return this
    }


    //子类重新赋值了，注意初始化顺序
    open fun initSizeParams() {
        baseBarAreaTop = templateBaseTop
        baseExpressTop = baseBarAreaTop + templateBarCodeAreaHeight
        baseReceiverTop = baseExpressTop + templateExpressCodeAreaHeight
        baseSenderTop = baseReceiverTop + templateReceiverAreaHeight
        baseInfoTop = baseSenderTop + templateSendAreaHeight
    }

    companion object {
        @JvmField
        var UNIT_RATIO = 8 //mm 单位到具体打印机的 系数
    }
}